@@ -1,36 +1,39 @@
+import logging
 import pkg_resources
-from pyramid.httpexceptions import HTTPBadRequest
+from pyramid.httpexceptions import HTTPBadRequest, HTTPFound
 from pyramid.response import Response
 from pyramid.security import remember, forget
-
 import browserid.errors
 
 
-def _check_csrf_token(request):
-    """Check the CSRF token from the request. Raises if invalid.
+logger = logging.getLogger(__name__)
 
-    Copied from pyramid.session.check_csrf_token in pyramid==1.4a2."""
-    if request.params.get('csrf_token') != request.session.get_csrf_token():
-        raise HTTPBadRequest('incorrect CSRF token')
+
+def verify_login(request):
+    """Verifies the assertion and the csrf token in the given request.
+
+    Returns the email of the user if everything is valid, otherwise raises
+    a HTTPBadRequest"""
+    verifier = request.registry['persona.verifier']
+    try:
+        data = verifier.verify(request.POST['assertion'])
+    except (ValueError, browserid.errors.TrustError) as e:
+        logger.info('Failed persona login: %s (%s)', e, type(e).__name__)
+        raise HTTPBadRequest('Invalid assertion')
+    return data['email']
 
 
 def login(request):
     """View to check the persona assertion and remember the user"""
-    _check_csrf_token(request)
-    verifier = request.registry['persona.verifier']
-    try:
-        data = verifier.verify(request.POST['assertion'])
-    except (ValueError, browserid.errors.TrustError):
-        raise HTTPBadRequest('invalid assertion')
-    headers = remember(request, data['email'])
-    return Response(headers=headers)
+    email = verify_login(request)
+    headers = remember(request, email)
+    return HTTPFound(request.POST['came_from'], headers=headers)
 
 
 def logout(request):
     """View to forget the user"""
-    _check_csrf_token(request)
     headers = forget(request)
-    return Response(headers=headers)
+    return HTTPFound(request.POST['came_from'], headers=headers)
 
 
 def forbidden(request):
